"""
Main loop for the Smart Doorbell system.

This script coordinates sensor input, handles notification triggers,
and displays relevant messages on the LCD.

Can run in two modes:
- MOCK_MODE = True:  uses fake sensors for dev/testing
- MOCK_MODE = False: uses real Raspberry Pi hardware

"""
import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
# ensures that it is run as a package
import time

from doorbell.config import (
    SCAN_INTERVAL, LIGHT_THRESHOLD, LIGHT_TRIGGER_DELAY,
    MOTION_TRIGGER_DELAY, NO_MOTION_RELEASE_DELAY, MOTION_COOLDOWN, PREFERENCES
)

from mocks.mock_loop import mock_telemetry_loop

from doorbell.sensors import (
    initialise_sensors, read_button, read_light, read_motion
)

from doorbell.display import set_display
from doorbell.notifier import notify
from doorbell.utils import log_event
from doorbell.mqtt_client import initialise_mqtt, send_data


# === Global state variables ===

# Light handling
light_dark_counter = 0
night_mode = False

# Motion handling
motion_counter = 0
no_motion_counter = 0
motion_detected = False
last_motion_time = 0

# Button handling
last_button_state = 1  # 1 = not pressed, 0 = pressed



#  === Core logic functions ===

def detect_button():
    """
    Check the current state of the doorbell button.
    If pressed, triggers notification and LCD display.

    """
    global last_button_state
    state = read_button()

    if state == 0 and last_button_state == 1:
        # Button just pressed
        send_data("Button Pressed")
        log_event("EVENT", "Button Pressed")
        set_display("Ringing...")
        notify(PREFERENCES)
        time.sleep(2)
        set_display("Smart Doorbell Ready")
        send_data("Button",{"mode": 0})

    # Update last state to detect edge case
    last_button_state = state

def detect_light():
    """
    Monitor ambient light levels and toggle night mode if needed.
    Uses rolling dark counter to prevent false positives.

    """
    global light_dark_counter, night_mode
    level = read_light()

    if level < LIGHT_THRESHOLD:
        light_dark_counter += 1
        if light_dark_counter >= LIGHT_TRIGGER_DELAY and not night_mode:
            night_mode = True
            log_event("EVENT", "Night mode ON")
            send_data("light_level",{"light":level})
            send_data("light_mode",{"mode": 1})
    else:
        light_dark_counter = 0
        if night_mode:
            night_mode = False
            log_event("EVENT", "Daylight detected")
            send_data("light_level",{"motion":level})
            send_data("light_mode",{"mode": 0})

def detect_motion():
    """
    Track motion events with a delay threshold and cooldown.
    Ensures motion is significant before alerting.

    """
    global motion_detected, motion_counter, no_motion_counter, last_motion_time
    motion = read_motion()
    now = time.time()

    if motion:
        motion_counter += 1
        no_motion_counter = 0
        if motion_counter >= MOTION_TRIGGER_DELAY and not motion_detected:
            if now - last_motion_time > MOTION_COOLDOWN:
                motion_detected = True
                last_motion_time = now
                log_event("EVENT", "Motion detected")
                send_data("motion",{"motion":1})
    else:
        if motion_detected:
            no_motion_counter += 1
            if no_motion_counter >= NO_MOTION_RELEASE_DELAY:
                motion_detected = False
                motion_counter = 0
                log_event("EVENT", "Motion ended")
                send_data("motion",{"motion":0})
        else:
            motion_counter = 0
            no_motion_counter = 0



# === Main application loop ===

# Main Script
def main_loop():
    """
    Starts the smart doorbell system.
    Continuously checks sensors and triggers appropriate actions.

    """
    log_event("SYSTEM", "Smart Doorbell Starting")
    set_display("Smart Doorbell Ready")

    try:
        while True:
            detect_button()
            detect_light()
            detect_motion()
            time.sleep(SCAN_INTERVAL)

    except KeyboardInterrupt:
        # Graceful shutdown on Ctrl+C
        set_display("System Halted")
        log_event("SYSTEM", "Shutdown triggered by keyboard")


# Entry Point
if __name__ == "__main__":
    initialise_sensors()
    initialise_mqtt()
    mock_telemetry_loop()
    #main_loop()
