# mqtt_client.py

import json
import time
from doorbell.config import (
    MOCK_MQTT, MQTT_BROKER, MQTT_PORT, MQTT_TOPIC, ACCESS_TOKEN
)
from doorbell.utils import log_event

if MOCK_MQTT:
    import mocks.mock_mqtt as mqtt
else:
    import paho.mqtt.client as mqtt

client = None

def on_connect(client, userdata, flags, rc, properties=None):
    if rc == 0:
        log_event("MQTT", "Connected to broker")
        client.subscribe("v1/devices/me/attributes/response/+")
        client.subscribe("v1/devices/me/attributes")
    else:
        log_event("ERROR", f"MQTT connect failed with code {rc}")

def on_message(client, userdata, msg):
    topic = msg.topic
    print(f"Received message on topic: {msg.topic}")
    if "attributes" in topic:
        try:
            payload = json.loads(msg.payload.decode())
            log_event("MQTT", f"Received shared attribute update: {payload}")

            from doorbell import config
            updated = {}

            for key in ["Audio", "Visual", "Thingsboard"]:
                if key in payload:
                    config.PREFERENCES[key] = payload[key]
                    updated[key] = payload[key]
                    log_event("CONFIG", f"{key} set to {payload[key]}")

            if updated:
                feedback = {
                    "audio_enabled": config.PREFERENCES["Audio"],
                    "visual_enabled": config.PREFERENCES["Visual"],
                    "thingsboard_enabled": config.PREFERENCES["Thingsboard"]
                }
                client.publish("v1/devices/me/attributes", json.dumps(feedback))
                log_event("MQTT", f"Reported updated preferences: {feedback}")

            if "attributes/response" in topic:
                shared_attrs = json.loads(msg.payload.decode()).get("shared", {})
                for key in ["Audio", "Visual", "Thingsboard"]:
                    if key in shared_attrs:
                        config.PREFERENCES[key] = shared_attrs[key]
                        log_event("CONFIG", f"{key} set to {shared_attrs[key]} (initial load)")



        except Exception as e:
            log_event("ERROR", f"Failed to process attribute update: {e}")

def initialise_mqtt():
    global client
    try:
        client = mqtt.Client(protocol=mqtt.MQTTv311)
        client.username_pw_set(ACCESS_TOKEN)
        client.on_connect = on_connect
        client.on_message = on_message
        client.connect(MQTT_BROKER, MQTT_PORT, 60)
        client.loop_start()
        time.sleep(1)

        client.publish("v1/devices/me/attributes/request/1", json.dumps({
            "sharedKeys": "Audio,Visual,Thingsboard"
        }))

    except Exception as e:
        log_event("ERROR", f"MQTT connection failed: {e}")

def send_data(event_type, extra_data=None):
    global client
    if not client:
        return
    try:
        payload = { "event": event_type }
        if extra_data:
            payload.update(extra_data)
        client.publish(MQTT_TOPIC, json.dumps(payload), qos=1)
        log_event("MQTT", f"Published: {payload}")
    except Exception as e:
        log_event("ERROR", f"MQTT publish failed: {e}")
